import logging

import click
from dotenv import load_dotenv
from waitress import serve

from plastron.web import create_app, __version__

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


@click.command()
@click.option(
    '--listen',
    default='0.0.0.0:5000',
    help='Address and port to listen on. Default is "0.0.0.0:5000".',
    metavar='[ADDRESS]:PORT',
)
@click.option(
    '-c', '--config-file',
    type=click.Path(exists=True),
    help='Configuration file',
    required=True,
)
def run(listen: bool, config_file: str):
    load_dotenv()
    server_identity = f'plastrond-http/{__version__}'
    logger.info(f'Starting {server_identity}')
    try:
        serve(
            app=create_app(config_file),
            listen=listen,
            ident=server_identity,
        )
    except (OSError, RuntimeError) as e:
        logger.error(f'Exiting: {e}')
        raise SystemExit(1) from e


if __name__ == "__main__":
    run()



import importlib.metadata
import logging
import os
import urllib.parse
from argparse import Namespace
from pathlib import Path

import yaml
from flask import Flask, url_for
from werkzeug.exceptions import NotFound, HTTPException

from plastron.web.flask_problem import problem_detail_response
from plastron.context import PlastronContext
from plastron.jobs import JobError, JobConfigError, JobNotFoundError, Jobs
from plastron.jobs.importjob import ImportJob
from plastron.utils import envsubst
from plastron.web.blueprints import activitystream_blueprint, resources_blueprint

__version__ = importlib.metadata.version('plastron-web')

logger = logging.getLogger(__name__)


def job_url(job_id):
    return url_for('show_job', _external=True, job_id=job_id)


def items(log):
    return {
        'count': len(log),
        'items': [c for c in log]
    }


def latest_dropped_items(job: ImportJob):
    latest_run = job.latest_run()
    if latest_run is None:
        return {}

    return {
        'timestamp': latest_run.timestamp,
        'failed': items(latest_run.failed_items),
        'invalid': items(latest_run.invalid_items)
    }


def create_app(config_file: str):
    app = Flask(__name__)
    with open(config_file, "r") as stream:
        config = envsubst(yaml.safe_load(stream))
        app.config['CONTEXT'] = PlastronContext(config=config, args=Namespace(delegated_user=None))
    jobs_dir = Path(os.environ.get('JOBS_DIR', 'jobs'))
    jobs = Jobs(directory=jobs_dir)
    app.register_blueprint(activitystream_blueprint)
    app.register_blueprint(resources_blueprint, url_prefix='/resources')

    def get_job(job_id: str):
        return jobs.get_job(ImportJob, urllib.parse.unquote(job_id))

    @app.route('/')
    def root():
        return {'version': __version__}

    @app.route('/jobs')
    def list_jobs():
        if not jobs_dir.exists():
            logger.warning(f'Jobs directory "{jobs_dir.absolute()}" does not exist; returning empty list')
            return {'jobs': []}
        job_ids = sorted(f.name for f in jobs_dir.iterdir() if f.is_dir())
        return {'jobs': [{'@id': job_url(job_id), 'job_id': job_id} for job_id in job_ids]}

    @app.route('/jobs/<path:job_id>')
    def show_job(job_id):
        try:
            job = get_job(job_id)
            job.load_config()
        except JobNotFoundError:
            logger.warning(f'Job {job_id} not found')
            raise NotFound
        except JobConfigError:
            logger.warning(f'Cannot open config file for job {job_id}')
            # TODO: more complete information in the response body?
            raise NotFound

        try:
            return {
                '@id': job_url(job_id),
                **vars(job.config),
                'runs': job.runs,
                'completed': items(job.completed_log),
                'dropped': latest_dropped_items(job),
                'total': job.get_metadata().total
            }
        except JobError as e:
            raise NotFound from e

    app.register_error_handler(HTTPException, problem_detail_response)

    return app



import pytest

from plastron.web import create_app


@pytest.fixture
def app_client(config_file_path, datadir, monkeypatch):
    def _create_app_client(jobs_dir):
        test_jobs_dir = datadir / jobs_dir
        monkeypatch.setenv('JOBS_DIR', str(test_jobs_dir))
        app = create_app(config_file_path)

        return app.test_client()

    return _create_app_client


def test_no_jobs(app_client):
    response = app_client('jobsempty').get('/jobs')
    assert response.status_code == 200
    data = response.get_json()
    assert 'jobs' in data
    assert len(data['jobs']) == 0


def test_jobs_found(app_client):
    response = app_client('jobs').get('/jobs')
    assert response.status_code == 200
    data = response.get_json()
    assert 'jobs' in data
    assert len(data['jobs']) == 5


def test_job_not_found(app_client):
    response = app_client('jobs').get('/jobs/FOOBAR')
    assert response.status_code == 404


def test_empty_job_dir(app_client):
    response = app_client('jobs').get('/jobs/noconfigfile')
    assert response.status_code == 404


def test_empty_config_file(app_client):
    response = app_client('jobs').get('/jobs/emptyconfigfile')
    assert response.status_code == 404


def test_valid_config_file_no_metadata(app_client):
    response = app_client('jobs').get('/jobs/nometadata')
    assert response.status_code == 404


def test_valid_job(app_client):
    response = app_client('jobs').get('/jobs/validjob')
    assert response.status_code == 200
    data = response.get_json()
    assert '@id' in data
    assert 'completed' in data
    assert data['completed']['count'] == 0
    assert len(data['completed']['items']) == 0
    assert 'dropped' in data
    assert len(data['dropped']) == 0
    assert 'runs' in data
    assert len(data['runs']) == 0
    assert data['access'] is None
    assert data['binaries_location'] == 'data'
    assert data['container'] == '/dc/2021/2'
    assert data['job_id'] == 'import-20210505143008'
    assert data['member_of'] == 'https://fcrepo.lib.umd.edu/fcrepo/rest/dc/2021/2'
    assert data['model'] == 'Item'
    assert data['total'] == 9


def test_valid_completed_job(app_client):
    response = app_client('jobs').get('/jobs/validcompletedjob')
    assert response.status_code == 200
    data = response.get_json()
    assert '@id' in data
    assert 'completed' in data
    assert data['completed']['count'] == 9
    assert len(data['completed']['items']) == 9
    assert 'dropped' in data
    # 3 keys: "failed", "invalid", "timestamp"
    assert len(data['dropped']) == 3
    assert 'failed' in data['dropped']
    assert 'invalid' in data['dropped']
    assert 'timestamp' in data['dropped']
    assert data['dropped']['timestamp'] == '20210505143008'
    assert 'runs' in data
    assert len(data['runs']) == 1
    assert data['runs'][0] == '20210505143008'
    assert data['access'] is None
    assert data['binaries_location'] == 'data'
    assert data['container'] == '/dc/2021/2'
    assert data['job_id'] == 'import-20210505143008'
    assert data['member_of'] == 'https://fcrepo.lib.umd.edu/fcrepo/rest/dc/2021/2'
    assert data['model'] == 'Item'
    assert data['total'] == 9



import json
from unittest.mock import MagicMock, ANY

import pytest

from plastron.context import PlastronContext
from plastron.handles import HandleServiceClient, HandleInfo
from plastron.repo import Repository
from plastron.repo.publish import PublishableResource


@pytest.fixture
def post_data():
    return {
        "@context": [
            "https://www.w3.org/ns/activitystreams",
            {
                "umdact": "http://vocab.lib.umd.edu/activity#",
                "Publish": "umdact:Publish",
                "PublishHidden": "umdact:PublishHidden",
                "Unpublish": "umdact:Unpublish"
            }
        ],
        "type": "Publish",
        "object": ["http://fcrepo-local:8080/fcrepo/rest/test/obj"]
    }


@pytest.fixture
def request_headers():
    mimetype = 'application/json'
    return {
        'Content-Type': mimetype,
        'Accept': mimetype
    }


@pytest.fixture
def mock_resource():
    mock_resource = MagicMock(spec=PublishableResource)
    mock_resource.read.return_value = mock_resource
    mock_handle = MagicMock(spec=HandleInfo)
    mock_resource.publish.return_value = mock_handle
    return mock_resource


@pytest.fixture
def mock_context(mock_resource):
    mock_repo = MagicMock(spec=Repository)
    mock_repo.__getitem__.return_value = mock_resource
    mock_context = MagicMock(spec=PlastronContext, repo=mock_repo, handle_client=MagicMock(spec=HandleServiceClient))
    mock_context.get_public_url.return_value = 'http://digital-local/foo'
    return mock_context


@pytest.mark.parametrize(
    ('input_json', 'expected_args'),
    [
        (
            # json input
            {
                "@context": [
                    "https://www.w3.org/ns/activitystreams",
                    {
                        "umdact": "http://vocab.lib.umd.edu/activity#",
                        "Publish": "umdact:Publish",
                        "PublishHidden": "umdact:PublishHidden",
                        "Unpublish": "umdact:Unpublish"
                    }
                ],
                "type": "Publish",
                "object": ["http://fcrepo-local:8080/fcrepo/rest/test/obj"]
            },
            # expected args
            {
                'force_hidden': False,
                'force_visible': False
            },
        ),
        (
            # json input
            {
                "@context": [
                    "https://www.w3.org/ns/activitystreams",
                    {
                        "umdact": "http://vocab.lib.umd.edu/activity#",
                        "Publish": "umdact:Publish",
                        "PublishHidden": "umdact:PublishHidden",
                        "Unpublish": "umdact:Unpublish"
                    }
                ],
                "type": "PublishHidden",
                "object": ["http://fcrepo-local:8080/fcrepo/rest/test/obj"]
            },
            # expected args
            {
                'force_hidden': True,
                'force_visible': False
            },
        ),
    ],
)
def test_new_activity_publish(
        app,
        app_client,
        mock_context,
        mock_resource,
        post_data,
        request_headers,
        input_json,
        expected_args,
):
    app.config['CONTEXT'] = mock_context

    url = '/inbox'
    response = app_client.post(url, data=json.dumps(input_json), headers=request_headers)
    assert response.status_code == 201

    mock_resource.publish.assert_called_once_with(handle_client=ANY, public_url=ANY, **expected_args)


@pytest.mark.parametrize(
    ('input_json', 'expected_args'),
    [
        (
            # json input
            {
                "@context": [
                    "https://www.w3.org/ns/activitystreams",
                    {
                        "umdact": "http://vocab.lib.umd.edu/activity#",
                        "Publish": "umdact:Publish",
                        "PublishHidden": "umdact:PublishHidden",
                        "Unpublish": "umdact:Unpublish"
                    }
                ],
                "type": "Unpublish",
                "object": ["http://fcrepo-local:8080/fcrepo/rest/test/obj"]
            },
            # expected args
            {
                'force_hidden': False,
                'force_visible': False
            },
        )
    ],
)
def test_new_activity_unpublish(
        app,
        app_client,
        mock_context,
        mock_resource,
        post_data,
        request_headers,
        input_json,
        expected_args,
):
    app.config['CONTEXT'] = mock_context

    url = '/inbox'
    response = app_client.post(url, data=json.dumps(input_json), headers=request_headers)
    assert response.status_code == 201

    mock_resource.unpublish.assert_called_once_with(**expected_args)


@pytest.mark.parametrize(
    'input_json',
    [
        # Missing type
        {
            "@context": [
                "https://www.w3.org/ns/activitystreams",
                {
                    "umdact": "http://vocab.lib.umd.edu/activity#",
                    "Publish": "umdact:Publish",
                    "PublishHidden": "umdact:PublishHidden",
                    "Unpublish": "umdact:Unpublish"
                }
            ],
            "object": ["http://fcrepo-local:8080/fcrepo/rest/test/obj"]
        },
        # Invalid type
        {
            "@context": [
                "https://www.w3.org/ns/activitystreams",
                {
                    "umdact": "http://vocab.lib.umd.edu/activity#",
                    "Publish": "umdact:Publish",
                    "PublishHidden": "umdact:PublishHidden",
                    "Unpublish": "umdact:Unpublish"
                }
            ],
            "type": "foo",
            "object": ["http://fcrepo-local:8080/fcrepo/rest/test/obj"]
        },
        # Missing target object
        {
            "@context": [
                "https://www.w3.org/ns/activitystreams",
                {
                    "umdact": "http://vocab.lib.umd.edu/activity#",
                    "Publish": "umdact:Publish",
                    "PublishHidden": "umdact:PublishHidden",
                    "Unpublish": "umdact:Unpublish"
                }
            ],
            "type": "Publish",
        }
    ],
)
def test_new_activity_invalid_input(app_client, post_data, request_headers, input_json):
    url = '/inbox'
    response = app_client.post(url, data=json.dumps(input_json), headers=request_headers)
    assert response.status_code == 400



import logging

from flask import Blueprint, request, current_app
from pyparsing import ParseException
from rdflib.plugins.sparql import prepareUpdate
from werkzeug.exceptions import BadRequest, UnsupportedMediaType, NotFound, InternalServerError

from plastron.web.flask_problem import ProblemDetailError
from plastron.models import get_model_from_name, ModelClassNotFoundError
from plastron.repo import RepositoryResource, RepositoryError

logger = logging.getLogger(__name__)
blueprint = Blueprint('resource', __name__)


@blueprint.route('/<path:resource_path>', methods=['PATCH'])
def update(resource_path):
    try:
        resource: RepositoryResource = current_app.config['CONTEXT'].repo.get_resource(resource_path).read()
    except RepositoryError as e:
        logger.error(str(e))
        raise NotFound from e

    if request.content_type != 'application/sparql-update':
        raise UnsupportedMediaType

    logger.info(f'Received request to update {resource.url}')

    sparql_text = request.data.decode()
    logger.debug(f'SPARQL Update Query: {sparql_text}')
    try:
        sparql_update = prepareUpdate(sparql_text, base=resource.url)
    except ParseException as e:
        raise SPARQLUpdateProblem(description=f'SPARQL Update Query parsing error: {e}')

    # Apply the update in-memory to the resource graph
    resource.graph.update(sparql_update)

    model_name = request.args.get('model', None)
    if model_name is not None:
        # validate according to the given content model
        try:
            model_class = get_model_from_name(model_name)
        except ModelClassNotFoundError as e:
            raise UnknownContentModel(model_name=e.model_name)

        # Validate the updated in-memory Graph using the model
        logger.info(f'Validating updated object using model "{model_name}"')
        obj = resource.describe(model_class)
        validation = obj.validate()
        if validation.ok:
            logger.info(f'Validation succeeded using model "{model_name}"')
        else:
            errors = {k: str(v) for k, v in validation.failures()}
            logger.error(f'Validation failed: {errors}')
            raise ContentModelValidationFailure(
                error_count=len(errors),
                validation_errors=errors,
                model_name=model_name,
                resource=resource.url,
            )

    try:
        resource.update()
        logger.info(f'Updated resource {resource.url}')
    except RepositoryError as e:
        logger.error(str(e))
        raise InternalServerError('Repository error')

    return '', 204


class SPARQLUpdateProblem(ProblemDetailError, BadRequest):
    name = 'SPARQL Update problem'


class UnknownContentModel(ProblemDetailError, BadRequest):
    name = 'Unrecognized content-model'
    description = '"{model_name}" is not a recognized content-model name'


class ContentModelValidationFailure(ProblemDetailError, BadRequest):
    name = 'Content-model validation failed'
    description = '{error_count} validation error(s) prevented update of {resource} with content-model {model_name}'



from http import HTTPStatus
from unittest.mock import MagicMock

import pytest

from plastron.context import PlastronContext
from plastron.models import ContentModeledResource
from plastron.rdfmapping.validation import ValidationResultsDict, ValidationFailure
from plastron.repo import RepositoryResource, Repository, RepositoryError


@pytest.fixture
def mock_valid_object():
    obj = MagicMock(spec=ContentModeledResource)
    obj.validate.return_value = ValidationResultsDict()
    return obj


@pytest.fixture
def mock_invalid_object():
    obj = MagicMock(spec=ContentModeledResource)
    obj.validate.return_value = ValidationResultsDict({'foo': ValidationFailure()})
    return obj


@pytest.fixture
def mock_resource():
    resource = MagicMock(spec=RepositoryResource)
    resource.read.return_value = resource
    return resource


@pytest.fixture
def mock_repo(mock_resource):
    repo = MagicMock(spec=Repository)
    repo.get_resource.return_value = mock_resource
    return repo


@pytest.fixture
def mock_context(mock_repo):
    return MagicMock(spec=PlastronContext, repo=mock_repo)


@pytest.fixture
def app_client_with_context(app_client, mock_context):
    app_client.application.config['CONTEXT'] = mock_context
    return app_client


def test_update_resource_not_found(mock_repo, app_client_with_context):
    mock_repo.get_resource.side_effect = RepositoryError

    response = app_client_with_context.patch(
        '/resources/foo',
        headers={'Content-Type': 'application/sparql-update'},
        data='DELETE {} INSERT {} WHERE {}',
        query_string={'model': 'Item'},
    )
    assert response.status_code == HTTPStatus.NOT_FOUND


def test_update_missing_content_type(app_client_with_context):
    response = app_client_with_context.patch(
        '/resources/foo',
    )
    assert response.status_code == HTTPStatus.UNSUPPORTED_MEDIA_TYPE


def test_update_wrong_content_type(app_client_with_context):
    response = app_client_with_context.patch(
        '/resources/foo',
        headers={'Content-Type': 'text/plain'}
    )
    assert response.status_code == HTTPStatus.UNSUPPORTED_MEDIA_TYPE


def test_update_unknown_content_model(app_client_with_context):
    response = app_client_with_context.patch(
        '/resources/foo',
        headers={'Content-Type': 'application/sparql-update'},
        data='DELETE {} INSERT {} WHERE {}',
        query_string={'model': 'FAKE_MODEL_THAT_DOES_NOT_EXIST'},
    )
    assert response.status_code == HTTPStatus.BAD_REQUEST
    problem_detail = response.json
    assert problem_detail['status'] == HTTPStatus.BAD_REQUEST
    assert problem_detail['title'] == 'Unrecognized content-model'
    assert 'is not a recognized content-model name' in problem_detail['details']


def test_update_bad_sparql(mock_resource, mock_valid_object, app_client_with_context):
    mock_resource.describe.return_value = mock_valid_object

    response = app_client_with_context.patch(
        '/resources/foo',
        headers={'Content-Type': 'application/sparql-update'},
        data='this is not SPARQL Update!!!',
        query_string={'model': 'Item'},
    )
    assert response.status_code == HTTPStatus.BAD_REQUEST
    problem_detail = response.json
    assert problem_detail['status'] == HTTPStatus.BAD_REQUEST
    assert problem_detail['title'] == 'SPARQL Update problem'
    assert 'SPARQL Update Query parsing error' in problem_detail['details']


def test_valid_update(mock_resource, mock_valid_object, app_client_with_context):
    mock_resource.describe.return_value = mock_valid_object

    response = app_client_with_context.patch(
        '/resources/foo',
        headers={'Content-Type': 'application/sparql-update'},
        data='DELETE {} INSERT {} WHERE {}',
        query_string={'model': 'Item'},
    )
    assert response.status_code == HTTPStatus.NO_CONTENT


def test_update_repository_error(mock_resource, mock_valid_object, app_client_with_context):
    mock_resource.describe.return_value = mock_valid_object
    mock_resource.update.side_effect = RepositoryError

    response = app_client_with_context.patch(
        '/resources/foo',
        headers={'Content-Type': 'application/sparql-update'},
        data='DELETE {} INSERT {} WHERE {}',
        query_string={'model': 'Item'},
    )
    assert response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR


def test_invalid_update(mock_resource, mock_invalid_object, app_client_with_context):
    mock_resource.describe.return_value = mock_invalid_object
    mock_resource.url = 'http://example.com/fcrepo/rest/foo'

    response = app_client_with_context.patch(
        '/resources/foo',
        headers={'Content-Type': 'application/sparql-update'},
        data='DELETE {} INSERT {} WHERE {}',
        query_string={'model': 'Item'},
    )
    assert response.status_code == HTTPStatus.BAD_REQUEST
    problem_detail = response.json
    assert problem_detail['status'] == HTTPStatus.BAD_REQUEST
    assert problem_detail['title'] == 'Content-model validation failed'
    assert 'validation error(s)' in problem_detail['details']



import json
from typing import Any, Dict

from werkzeug import Response
from werkzeug.exceptions import HTTPException


class ProblemDetailError(HTTPException):
    """Subclass of the Werkzeug `HTTPException` class that adds a `params`
    dictionary that `as_problem_detail()` uses to format the response details.
    The items in the `params` dictionary are also included as extension
    members in the JSON problem detail response.

    Individual applications should create their own application-specific
    subclasses of this class for exceptions that should result in a problem
    detail response. Those exceptions should also subclass one of the standard
    Werkzeug HTTP exceptions in the `werkzeug.exceptions` module, such as
    `NotFound` or `BadRequest`, in order to provide the correct HTTP response
    codes."""

    name: str
    """Used as the problem detail `title`."""
    description: str
    """Used as the problem detail `details`. The value is treated as a format
    string, and is filled in using the `params` dictionary."""

    def __init__(self, description=None, response=None, **params):
        super().__init__(description, response)
        self.params = params

    def as_problem_detail(self) -> Dict[str, Any]:
        """Format the exception information as a dictionary with keys as
        specified in the [RFC 9457](https://www.rfc-editor.org/rfc/rfc9457)
        JSON Problem Details format.

        | RFC 9457 Key | Attribute |
        |--------------|-----------|
        | `"status"`   | `code`    |
        | `"title"`    | `name`    |
        | `"details"`  | `description`, formatted using the `params` dictionary |

        The items in the `params` dictionary are also included in the problem
        details as [extension members](https://www.rfc-editor.org/rfc/rfc9457#name-extension-members).

        """
        return {
            'status': self.code,
            'title': self.name,
            'details': self.description.format(**self.params),
            **self.params,
        }


def problem_detail_response(e: HTTPException) -> Response:
    """Return a JSON Problem Detail ([RFC 9457](https://www.rfc-editor.org/rfc/rfc9457))
    for HTTP errors.

    This function is mainly intended to be registered as an error handler
    with a Flask app:

    ```python
    from flask import Flask
    from flask_problem import ProblemDetailError, problem_detail_response

    app = Flask(__name__)

    ...

    app.register_error_handler(ProblemDetailError, problem_detail_response)
    ```
    """
    # start with the correct headers and status code from the error
    response = e.get_response()
    # replace the body with JSON
    if isinstance(e, ProblemDetailError):
        response.data = json.dumps(e.as_problem_detail())
    else:
        response.data = json.dumps({'status': e.code, 'title': e.name, 'details': e.description})
    response.content_type = 'application/problem+json'
    return response



import json
import logging
from typing import List
from uuid import uuid4

from flask import Blueprint, current_app, jsonify, request
from rdflib import Graph

from plastron.namespaces import activitystreams, rdf, umdact
from plastron.repo.publish import PublishableResource

logger = logging.getLogger(__name__)

blueprint = Blueprint('activitystream', __name__, template_folder='templates')


@blueprint.route('/inbox', methods=['POST'])
def new_activity():
    try:
        activity = Activity(from_json=request.get_json())
        ctx = current_app.config['CONTEXT']
        for uri in activity.objects:
            resource: PublishableResource = ctx.repo[uri:PublishableResource].read()
            if activity.publish:
                resource.publish(
                    handle_client=ctx.handle_client,
                    public_url=ctx.get_public_url(resource),
                    force_hidden=activity.force_hidden,
                    force_visible=False,
                )
            elif activity.unpublish:
                resource.unpublish(
                    force_hidden=activity.force_hidden,
                    force_visible=False,
                )
        return {}, 201
    except ValidationError as e:
        logger.error(f'Exception: {e}')
        return jsonify({'error': str(e)}), 400
    except Exception as e:
        logger.error(f'Exception: {e}')
        return jsonify({'error': str(e)}), 500


class Activity:
    def __init__(self, from_json: str):
        self.id = uuid4()
        self._objects = []
        self._publish = False
        self._unpublish = False
        self._force_hidden = False
        g = Graph()
        g.parse(data=json.dumps(from_json), format='json-ld')

        for s, p, o in g:
            if activitystreams.object == p:
                self._objects.append(str(o))
            elif rdf.type == p:
                if o == umdact.Publish:
                    self._publish = True
                elif o == umdact.Unpublish:
                    self._unpublish = True
                elif o == umdact.PublishHidden:
                    self._publish = True
                    self._force_hidden = True
                else:
                    raise ValidationError(f'Invalid Activity type: {str(o)}')
        if not self.publish and not self.unpublish:
            raise ValidationError(f'Invalid JSON-LD provided: Type not specified.')
        if not self.objects:
            raise ValidationError(f'Invalid JSON-LD provided: Object(s) not specified.')

    def __str__(self):
        return self.id

    @property
    def publish(self) -> bool:
        return self._publish

    @property
    def unpublish(self) -> bool:
        return self._unpublish

    @property
    def force_hidden(self) -> bool:
        return self._force_hidden

    @property
    def objects(self) -> List[str]:
        return self._objects


class ValidationError(Exception):
    pass
